var vows = require( 'vows' ),
    assert = require( 'assert' ),
    equality = require( '../lib/equality' );

vows.describe( 'deep()' ).addBatch({

  'works': {

    topic: function () { return equality.deep; },

    'for atomic values': function ( deep ) {
      // numbers
      assert.isTrue( deep( 1, 1 ) );
      assert.isFalse( deep( 1, 2 ) );
      assert.isTrue( deep( 1.1, 1.1 ) );
      assert.isTrue( deep( 1.0, 1 ) );
      assert.isTrue( deep( 1, 1.0 ) );
      assert.isFalse( deep( 1.1, 1.2 ) );

      // numbers and booleans
      assert.isFalse( deep( 1, true ) );
      assert.isFalse( deep( true, 1 ) );
      assert.isFalse( deep( 1.1, true ) );
      assert.isFalse( deep( true, 1.1 ) );

      // numbers and strings
      assert.isFalse( deep( 1, 'one' ) );
      assert.isFalse( deep( 'one', 1 ) );
      assert.isFalse( deep( 1, '1' ) );
      assert.isFalse( deep( '1', 1 ) );
      assert.isFalse( deep( '1.0', 1.0 ) );
      assert.isFalse( deep( 1.0, '1.0' ) );

      // numbers and objects
      assert.isFalse( deep( 1, { a: 2 } ) );
      assert.isFalse( deep( { a: 2 }, 1 ) );
      assert.isFalse( deep( 1.1, {} ) );
      assert.isFalse( deep( {}, 1.1 ) );

      // numbers and arrays
      assert.isFalse( deep( 1, [] ) );
      assert.isFalse( deep( [], 1 ) );

      // booleans
      assert.isTrue( deep( true, true ) );
      assert.isFalse( deep( true, false ) );
      assert.isTrue( deep( false, false ) );

      // booleans and strings
      assert.isFalse( deep( true, 'true' ) );
      assert.isFalse( deep( 'true', true ) );
      assert.isFalse( deep( false, 'false' ) );
      assert.isFalse( deep( 'false', false ) );

      // booleans and objects
      assert.isFalse( deep( true, { a: 2 } ) );
      assert.isFalse( deep( { a: 2 }, true ) );

      // booleans and arrays
      assert.isFalse( deep( true, [] ) );
      assert.isFalse( deep( [], true ) );

      // strings
      assert.isTrue( deep( 'string', 'string' ) );
      assert.isFalse( deep( 'string', 'number' ) );

      // strings and objects
      assert.isFalse( deep( 'string', { a: 2 } ) );
      assert.isFalse( deep( { a: 2 }, 'string' ) );
      assert.isFalse( deep( ',a==2', { a: 2 } ) );

      // strings and arrays
      assert.isFalse( deep( 'string', [] ) );
      assert.isFalse( deep( [], 'string' ) );
    },

    'for null values': function ( deep ) {
      // two nulls
      assert.isTrue( deep( null, null ) );
      // numbers
      assert.isFalse( deep( null, 1 ) );
      assert.isFalse( deep( 1, null ) );
      // booleans
      assert.isFalse( deep( true, null ) );
      assert.isFalse( deep( null, true ) );
      // strings
      assert.isFalse( deep( 'string', null ) );
      assert.isFalse( deep( null, 'string' ) );
      // objects
      assert.isFalse( deep( null, {} ) );
      assert.isFalse( deep( {}, null ) );
      // arrays
      assert.isFalse( deep( [], null ) );
      assert.isFalse( deep( null, [] ) );

      // objects with nulls
      var X = { a: null, b: 10 },
          Y = { a: null, b: 10 },
          Z = { a: 10, b: null };

      assert.isTrue( deep( X, Y ) );
      assert.isFalse( deep( Y, Z ) );

      var A = { x: X, y: Y, z: Z },
          B = { x: X, y: Y, z: Z },
          C = { a: X };

      assert.isTrue( deep( A, B ) );
      assert.isFalse( deep( B, C ) );
      assert.isTrue( deep( A.x, C.a ) );

      // arrays with nulls
      assert.isTrue( deep( [ null, null ], [ null, null ] ) );
      assert.isFalse( deep( [], [ null ] ) );
    },

    'for empty values': function ( deep ) {
      var A = {},
          B = {},
          C = { a: 2 };

      assert.isTrue( deep( A, B ) );
      assert.isFalse( deep( B, C ) );

      var D = [],
          E = [],
          F = [ 1 ];

      assert.isTrue( deep( D, E ) );
      assert.isFalse( deep( E, F ) );
      
      var G = '',
          H = '',
          I = 'a';

      assert.isTrue( deep( G, H ) );
      assert.isFalse( deep( H, I ) );

      assert.isTrue( deep( [ G, H, G ], [ H, H, H ] ) );
      assert.isTrue( deep( { a: G, b: H }, { a: H, b: G } ) );
    },

    'for objects with objects': function ( deep ) {
      // deep
      var A = { a: { b: 'c' }, d: { e: 'f' } },
          B = { a: { b: 'c' }, d: { e: 'f' } },
          C = { x: 10, y: { e: 'f' } };

      assert.isTrue( deep( A, B ) );
      assert.isFalse( deep( B, C ) );
      assert.isTrue( deep( A.d, C.y ) );

      // deeper!
      var X = { a: A, b: B, c: C },
          Y = { a: A, b: B, c: C },
          Z = { a: C, b: A, c: B };

      assert.isTrue( deep( X, Y ) );
      assert.isFalse( deep( Y, Z ) );
      assert.isTrue( deep( X.a, Z.b ) );

      // wtf is going on...
      var U = { x: X, y: Y, z: Z },
          V = { x: X, y: Y, z: Z },
          W = { x: Z, y: X, z: Y };

      assert.isTrue( deep( U, V ) );
      assert.isFalse( deep( V, W ) );
      assert.isTrue( deep( V.x, W.y ) );
    },

    'for arrays': function ( deep ) {
      var A = { a: { b: 'c' }, d: { e: 'f' } },
          B = { a: { b: 'c' }, d: { e: 'f' } },
          C = { x: 10, y: { e: 'f' } },
          D = { a: A, b: B, c: C },
          E = { a: A, b: B, c: C },
          F = { a: C, b: A, c: B };

      var U = [ A, B, C, D, E, F ],
          V = [ A, B, C, D, E, F ],
          W = [ F, E, D, C, B, A ];

      assert.isTrue( deep( U, V ) );
      assert.isFalse( deep( V, W ) );

      var X = [ U, V, W ],
          Y = [ U, V, W ],
          Z = [ W, V, U ];

      assert.isTrue( deep( X, Y ) );
      assert.isFalse( deep( Y, Z ) );
      assert.isTrue( deep( Y[1], Z[1] ) );

      var G = { a: 10, b: X },
          H = { a: 10, b: X },
          I = { a: X, b: 10 };

      assert.isTrue( deep( G, H ) );
      assert.isFalse( deep( H, I ) );
      assert.isTrue( deep( H.b, I.a ) );
    }

  }

}).export( module );
