var vows = require( 'vows' ),
    assert = require( 'assert' ),
    equality = require( '../lib/equality' );

vows.describe( 'members()' ).addBatch({

  'works': {

    topic: function () { return equality.members; },

    'with arrays of atomic values': function ( members ) {
      // numbers
      var A = [ 1, 2, 3, 4, 1 ],
          B = [ 4, 3, 1, 2, 1 ],
          C = [ 5, 6, 7 ];

      assert.isTrue( members( A, B ) );
      assert.isTrue( members( B, A ) );
      assert.isFalse( members( B, C ) );

      // strings
      var D = [ 'a', 'b', 'c', 'a' ],
          E = [ 'a', 'c', 'b', 'a' ],
          F = [ 'd', 'e', 'f' ];

      assert.isTrue( members( D, E ) );
      assert.isTrue( members( E, D ) );
      assert.isFalse( members( E, F ) );

      // booleans
      var G = [ true, false, true ],
          H = [ false, true, true ],
          I = [ false, false, false ];

      assert.isTrue( members( G, H ) );
      assert.isTrue( members( H, G ) );
      assert.isFalse( members( H, I ) );

      // mixed
      var J = [ 'a', 1, true, true, 1, 'b', 'a', 4 ],
          K = [ 1, true, 1, true, 'b', 'a', 4, 'a' ],
          L = [ 1, true, 1, true, 'b', 'a', 5, 'a' ];

      assert.isTrue( members( J, K ) );
      assert.isFalse( members( K, L ) );
    },

    'with nested arrays of atomic values': function ( members ) {
      // numbers
      var A = [ 1, 2, 3, 4, 1 ],
          B = [ 4, 3, 1, 2, 1 ],
          C = [ 5, 6, 7 ];

      var D = [ A, B, C, B, A ],
          E = [ A, B, C, B, A ],
          F = [ B, B, B, B, B ];

      assert.isTrue( members( D, E ) );
      assert.isFalse( members( E, F ) );

      // string
      var G = [ 'a', 'b', 'c', 'a' ],
          H = [ 'a', 'c', 'b', 'a' ],
          I = [ 'd', 'e', 'f' ];

      var J = [ G, H, I, H, G, I ],
          K = [ G, H, I, H, G, I ],
          L = [ I, I, I, I, I, I ];

      assert.isTrue( members( J, K ) );
      assert.isFalse( members( K, L ) );

      // booleans
      var M = [ true, false, true ],
          N = [ false, true, true ],
          O = [ false, false, false ];

      var P = [ M, N, O, M, N, O ],
          Q = [ M, N, O, M, N, O ],
          R = [ M, M, M, M, M, M ];

      assert.isTrue( members( P, Q ) );
      assert.isFalse( members( Q, R ) );

      // mixed
      var S = [ 'a', 1, true, true, 1, 'b', 'a', 4 ],
          T = [ 1, true, 1, true, 'b', 'a', 4, 'a' ],
          U = [ 1, true, 1, true, 'b', 'a', 5, 'a' ];

      var V = [ S, T, U, S ],
          W = [ S, T, U, S ],
          X = [ S, S, T, S ];

      assert.isTrue( members( V, W ) );
      assert.isFalse( members( W, X ) );
    }

  }

}).export( module );
